#include <allegro.h>

#ifndef __MAIN_H
#define __MAIN_H

#define FPS 60

#define TRUE 	-1
#define FALSE	0

#define LEFT	0
#define RIGHT	1

#define IN_MAP			0
#define IN_CHOOSEMENU	1

#define DO_UP		0
#define DO_DOWN		1
#define DO_LEFT		2
#define DO_RIGHT	3
#define DO_JUMP		4
#define DO_DASH		5
#define DO_SHOOT	6

#define SLOPE_0		0
#define SLOPE_90	1
#define SLOPE_45L	2
#define SLOPE_45R	3

#define DOING_WALKING 		0
#define DOING_JUMPING 		1
#define DOING_FALLING 		2
#define DOING_DASHING 		3
#define DOING_DUCKING 		4
#define DOING_SLIDING 		5
#define DOING_STANDING		6
#define DOING_HURTING 		7
#define DOING_SLIDEJUMPING 	8

#define PLAYER_MEGAMANX		0
#define PLAYER_ZERO			1
#define PLAYER_KNUCKLES		98
#define PLAYER_TESTGUY		99

typedef struct _spriteset
{
	int xsize, ysize;
	int numofsprites;

	BITMAP* sprites[20];

}spriteset;

typedef struct _anim
{
	spriteset Frames;
	char * FramePath;
	int NumOfFrames;
    int LoopFrame;
	int Speed;

	int xsize, ysize;
}Animation;

typedef struct _playerinfo
{
	char *Name;

	int WalkSpeed, FallSpeed, JumpSpeed;
	int MaxJumpHeight;
	int DashDistance;
	int Height;
	int Width;
}PlayerInfo;

typedef struct _player
{
	char Character;
	int CurrentAction;
	int CurrentFrame;
	PlayerInfo PlayerData;
	int Facing;
	int hp;

	int xpos,ypos;
	int height,width;

	int JumpPressed;
	int ShootPressed;
	int DashPressed;

	int DashPos;
	int Shooting;
	int ShootingPos;
	int ShootingDur;

	int DashJump;
	int JumpHeight;

	int SpawnDustPos;
	int SpawnDustSpeed;

	int AnimPos;
}player;

/* From vars.c */
extern int BackgroundOffset;
extern int ExitSystem;
extern int ExitMenu;
extern char Controls[7];
extern BITMAP* TileBay[256];
extern BITMAP* background;
extern BITMAP* Buffer;
extern unsigned int MenuBackground;
extern unsigned int MenuCursor;
extern int mapxsize, mapysize;
extern int CurrentlyIn;
extern unsigned char* EntityLayer;
extern unsigned char* ForegroundLayer;
extern unsigned char* SlopeTable;
extern player Players[2];
extern PlayerInfo MegamanXData;
extern PlayerInfo ZeroData;
extern int camerax, cameray;
extern int ColorDepth;
extern volatile int aspeed;
extern volatile int gspeed;
extern volatile int last_fps;
extern volatile int frame_counter;

/* From system.c */
extern void SystemInit();
extern void UpdateScreen( int value1, int value2 );
//extern void bin_to_texture(char *filename, uint32 tex, int w, int h);
//extern void DrawLCD(const char *vmu_icon);
extern void DrawSprite( BITMAP* bmp, int xpos, int ypos, int xsize, int ysize, int flip );
extern void DrawFrame( Animation Anim, int playerindex );

/* From timer.c */
extern void frame_control(void);

/* From engine.c */
extern void GetInput();
extern void ProcessPlayers();
extern void ProcessMegamanX( int playerindex );
extern void DrawPlayers();
extern void DrawMegamanX( int playerindex );
extern void DrawZero( int playerindex );
extern void DrawKnuckles( int playerindex );
extern int MovePlayer( int playerindex, int xpos, int ypos );
extern int CollisionAt( int playerindex, int xpos, int ypos );
extern int InAir( int playerindex );
extern void ChangePlayerAction( int playerindex, int Action, int resetanim );
extern int TouchingWall( int playerindex, int DirCheck );
extern void SetupPlayerInfo();
extern void LoadAnimations();
extern int UpdateAnim( Animation Anim, int playerindex );
extern void UpdateMegamanXAnimation( int playerindex );
extern void UpdateZeroAnimation( int playerindex );
extern void UpdateKnucklesAnimation( int playerindex );
extern void SetCamera( int playerindex );
extern void EngineLoop();
extern void SetCharacter( int playerindex, int player );
extern void UpdateAnimations();

/* From map.c */
extern char* CreateLayer( int xsize, int ysize );
extern void ResizeLayer( int xsize, int ysize, char* layer );
extern void DestroyLayer( char* layer );
extern void DrawLayer( unsigned char * layer, int camx, int camy, int xsize, int ysize );
extern void DrawTile( int xpos, int ypos, int index, int scale );
extern void ChangeTile( int x, int y, int tileindex );
extern void SaveMap( char* filename );
extern void LoadMap( char* filename );
extern void LoadTLE( char* filename );

/* From set.c */
extern spriteset LoadSpriteSet( char * filename );
extern void DestroySpriteSet( spriteset s );
extern spriteset CreateSpriteSet( int xsize, int ysize, int sprites );

/* From menu.c */
extern int ChooseMenu();

/* From timer.c */


/* Animations */
extern Animation MegamanXWalking;
extern Animation MegamanXStanding;
extern Animation MegamanXJumping;
extern Animation MegamanXSlideJumping;
extern Animation MegamanXFalling;
extern Animation MegamanXSliding;
extern Animation MegamanXDashing;
extern Animation MegamanXDucking;

extern Animation MegamanXSWalking;
extern Animation MegamanXSStanding;
extern Animation MegamanXSJumping;
extern Animation MegamanXSSlideJumping;
extern Animation MegamanXSFalling;
extern Animation MegamanXSSliding;
extern Animation MegamanXSDashing;
extern Animation MegamanXSDucking;

extern Animation ZeroWalking;
extern Animation ZeroStanding;
extern Animation ZeroJumping;
extern Animation ZeroSlideJumping;
extern Animation ZeroFalling;
extern Animation ZeroSliding;
extern Animation ZeroDashing;
extern Animation ZeroDucking;

extern Animation ZeroSWalking;
extern Animation ZeroSStanding;
extern Animation ZeroSJumping;
extern Animation ZeroSSlideJumping;
extern Animation ZeroSFalling;
extern Animation ZeroSSliding;
extern Animation ZeroSDashing;
extern Animation ZeroSDucking;

extern Animation TestGuy;

extern Animation KnucklesWalking;
extern Animation KnucklesStanding;
extern Animation KnucklesJumping;
extern Animation KnucklesSlideJumping;
extern Animation KnucklesFalling;
extern Animation KnucklesSliding;
extern Animation KnucklesDashing;
extern Animation KnucklesDucking;

extern Animation KnucklesSWalking;
extern Animation KnucklesSStanding;
extern Animation KnucklesSJumping;
extern Animation KnucklesSSlideJumping;
extern Animation KnucklesSFalling;
extern Animation KnucklesSSliding;
extern Animation KnucklesSDashing;
extern Animation KnucklesSDucking;

#endif

